@@ -173,7 +173,7 @@ def read_rism_patch(filename, section="rism_patch"):
     return ff
 
 
-def read_system(
+def read_molecule(
     filename,
     geometry_section="geometry",
     interaction_section="interactions",
@@ -325,6 +325,33 @@ def write_array1(filename, grid, matrix, header=None):
             fp.write(F"{matrix[k]}  ")
             fp.write("\n")
 
+# def sim_to_netcdf(sim):
+#
+#     rdims = ["site", "rgrid"]
+#     kdims = ["site", "kgrid"]
+#     data_r = dict(h_r=(rdims, sim.h_r),
+#                   g_r=(rdims, sim.g_r),
+#                   gl_r=(rdims, sim.gl_r),
+#                   vs_r=(rdims, sim.vs_r),
+#                   cs_r=(rdims, sim.cs_r),
+#                   xi_r=(rdims, sim.xi_r),
+#                   epot_r=(rdims, sim.epot_r),
+#                   vl_r=(rdims, sim.vl_r)
+#                   )
+#
+#     data_k = dict(h_k=(kdims, sim.h_k),
+#                   gl_k=(kdims, sim.gl_k),
+#                   vl_k=(kdims, sim.vl_k),
+#                   cs_k=(kdims, sim.cs_k),
+#                   epot_k=(kdims, sim.epot_k)
+#                   )
+#
+#     coords = dict(site=sim.solvent.aname, rgrid=sim.rgrid, kgrid=sim.kgrid)
+#     ds = xr.Dataset(data_vars=data_r | data_k, coords=coords, attrs=dict(solute=solute))
+#     print(ds)
+#     ds.to_netcdf("test.nc", engine='h5netcdf')
+
+
 if __name__ == "__main__":
     filename = "/Users/marat/codes/cdft-dev/data/input.dat"
     print(read_solute(filename))
