@@ -3,6 +3,11 @@
 """
 Main CDFT module
 """
+import io
+import logging
+from contextlib import redirect_stdout
+
+logging.getLogger('matplotlib').setLevel(logging.WARNING)
 import os
 import sys
 from types import SimpleNamespace
@@ -18,10 +23,12 @@ from cdftpy.cdft1d.io_utils import print_simulation
 from cdftpy.cdft1d.potential import compute_lj_potential_mod
 from cdftpy.cdft1d.rdf import analyze_rdf_peaks_sim
 from cdftpy.cdft1d.rdf import write_rdf_sim
-from cdftpy.cdft1d.solvent import Solvent, solvent_model_locate
+from cdftpy.cdft1d.solvent import Solvent, solvent_model_locate, Molecule1
 from cdftpy.utils.units import R
 from cdftpy import __version__
 
+
+
 HEADER = F"""
 ==================================
 1D RSDFT PROGRAM
@@ -41,8 +48,12 @@ DEFAULT_PARAMS = dict(diis_iterations=2, tol=1.0e-9, output_rate=10, max_iter=20
 DG_STRING = "d_g"
 
 
-def rsdft_1d(solute, solvent, params=None, quiet=False):
+def rsdft_1d(solute, solvent, params=None, quiet=False, gr_guess=None):
 
+    if solvent.nv > 2:
+        print("RSDFT method is currently restricted to diatomic liquids")
+        print("As an alternative you can try RISM")
+        sys.exit(1)
     if quiet:
         sys.stdout = open(os.devnull, 'w')
 
@@ -54,7 +65,7 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
     sig_v = solvent.sigma
     eps_v = solvent.eps
     qv = solvent.charge
-    s_k = solvent.s_k
+
 
     if params is None:
         params = DEFAULT_PARAMS
@@ -69,6 +80,12 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
     if "temp" not in params:
         params["temp"] = solvent.temp
 
+    if "rmax" in params:
+        rmax = params["rmax"]
+        solvent.extend(rmax)
+    else:
+        rmax = solvent.ifft.rgrid[-1]
+
     temp = float(params["temp"])
     beta = 1.0 / (kb * temp)
 
@@ -93,7 +110,8 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
     # total short range potential
     vs_r = vcs_r + vlj_r
 
-    # molecular structure factor
+    # structure factors
+    s_k = solvent.s_k
     sm = solvent.sm_k(kgrid)
 
     inv_sm = np.linalg.inv(sm.transpose(2, 0, 1))
@@ -102,7 +120,10 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
 
     gl_r, gl_k = compute_gamma_long_range(ifft, hbar, sm, vl_k, vl_r, zeta)
 
-    g_r = np.zeros(hbar[0].shape)
+    if gr_guess is None:
+        g_r = np.zeros(hbar[0].shape)
+    else:
+        g_r = gr_guess
 
     print("")
     print_banner("   Self-consistent cycle     ")
@@ -151,10 +172,13 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
         )
         # sys.exit(1)
 
+    xi_r = compute_correlation_hole(ifft, sm, f_r, f_k)
 
     print("\n")
     print(f"{'Total Free Energy ':<30} {fe_tot:>12.6f}")
 
+    cs_r = np.apply_along_axis(ifft.to_rspace, 1, cs_k)
+
     fe_tot, fe_extra = compute_free_energy(beta, rho_0, ifft, vl_r, g_r, h_r, cs_k)
 
     fe_gas, fe_inter = fe_extra
@@ -164,6 +188,7 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
 
     sys.stdout = sys.__stdout__
     return SimpleNamespace(
+        method=F"rsdft v{__version__}",
         solute=SimpleNamespace(**solute),
         solvent=solvent,
         ifft=ifft,
@@ -174,8 +199,10 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
         gl_k=gl_k,
         vl_r=vl_r,
         vl_k=vl_k,
+        vs_r=vs_r,
         vcs_r=vcs_r,
         h_r=h_r,
+        xi_r=xi_r,
         g_r=g_r,
         fe_tot=fe_tot,
         fe_gas=fe_gas,
@@ -183,7 +210,10 @@ def rsdft_1d(solute, solvent, params=None, quiet=False):
         phi_r=-(g_r + vl_r) / beta,
         epot_r=epot_r,
         epot_k=epot_k,
-        beta=beta
+        cs_k = cs_k,
+        cs_r = cs_r,
+        beta=beta,
+        rmax=rmax
     )
 
 
@@ -217,6 +247,18 @@ def compute_delta_h(ifft, sm, f_r, f_k):
 
     return dh_r
 
+def compute_correlation_hole(ifft, sm, f_r, f_k):
+    dd = sm - 1.0
+
+    dd_fk = np.einsum("abn,bn->an", dd, f_k)
+
+    dd_fr = np.apply_along_axis(ifft.to_rspace, 1, dd_fk)
+
+    xi_r = np.sum(f_r, axis=0) + dd_fr - f_r
+
+
+    return xi_r
+
 
 def compute_c_k(ifft, sm, h_r, g_r):
     h_k = np.apply_along_axis(ifft.to_kspace, 1, h_r)
@@ -274,12 +316,13 @@ def compute_free_energy(beta, rho_0, ifft, vl_r, g_r, h_r, c_k) -> float:
     if nsites != 2:
         raise ValueError
 
-    phi_r = -(g_r + vl_r) / beta
+    c_r = np.apply_along_axis(ifft.to_rspace, 1, c_k)
+    fe0 = -rho_0 * ifft.integrate_rspace(c_r) / nsites / beta
 
-    fe0 = -rho_0 * np.sum(c_k[:, 0]) / nsites / beta
     # alternative way of calculating fe0
-    # c_r = np.apply_along_axis(ifft.to_rspace, 1, c_k)
-    # fe0_1 = -rho_0 * ifft.integrate_rspace(c_r) / nsites / beta
+    # fe0 = -rho_0 * np.sum(c_k[:, 0]) / nsites / beta
+
+    phi_r = -(g_r + vl_r) / beta
 
     fe1 = 0.5*rho_0*ifft.integrate_rspace(phi_r * h_r)
     fe_tot = fe0 - fe1
@@ -289,16 +332,53 @@ def compute_free_energy(beta, rho_0, ifft, vl_r, g_r, h_r, c_k) -> float:
 
 if __name__ == "__main__":
 
+    import time
+
     # load solvent model
-    solvent_name = "s2"
+    solvent_name = "hcl0"
     filename = solvent_model_locate(solvent_name)
     solvent = Solvent.from_file(filename)
+    # solvent1 = copy.deepcopy(solvent)
+    #
+    # solvent1.extend(500)
+
+    pass
 
     solute = dict(name="Cl", charge=-1.0, sigma=4.83, eps=0.05349244)
-    params = dict(diis_iterations=2, tol=1.0e-7, max_iter=200)
+    m = Molecule1(aname=["Cl"], charge=[-1.0], sigma=[4.83], eps=[0.05349244])
+    # params = dict(diis_iterations=2, tol=1.0e-7, max_iter=500)
+    # sim = rsdft_1d(solute, solvent, params=params)
 
-    sim = rsdft_1d(solute, solvent, params=params)
 
-    analyze_rdf_peaks_sim(sim)
 
-    write_rdf_sim(sim)
+    rmax = solvent.ifft.rgrid[-1]
+    params = dict(diis_iterations=2, tol=1.0e-7, max_iter=500)
+
+    start_time = time.process_time()
+    f = io.StringIO()
+    with redirect_stdout(f):
+        sim = rsdft_1d(solute, solvent, params=params)
+    s = f.getvalue()
+    print(s)
+    # t = time.process_time() - start_time
+    # print(rmax, sim.fe_tot,t)
+    #
+    # analyze_rdf_peaks_sim(sim)
+
+    # for rmax in np.linspace(100, 1900, 10):
+    #     params["rmax"] = rmax
+    #     start_time = time.process_time()
+    #     sim = rsdft_1d(solute, solvent, params=params, quiet=True)
+    #     t = time.process_time() - start_time
+    #     print(rmax, sim.fe_tot,t)
+
+
+    # analyze_rdf_peaks_sim(sim)
+
+
+
+    # solvent.extend(1000)
+    # sim = rsdft_1d(solute, solvent, params=params)
+    #
+    # analyze_rdf_peaks_sim(sim)
+    # write_rdf_sim(sim)
